//+------------------------------------------------------------------+
//|                                                        Aroon.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
//---- copyright
#property copyright "2010, MetaQuotes Software Corp."
//---- link to the author's site
#property link      "http://www.mql5.com"
//---- version number
#property version   "1.00"
//---- plot indicator in the separate window
#property indicator_separate_window
//---- two buffers are used
#property indicator_buffers 2
//---- two plots are used
#property indicator_plots   2
//+----------------------------------------------+
//|  Bull indicator plot parameters              |
//+----------------------------------------------+
//---- drawing style = line
#property indicator_type1   DRAW_LINE
//---- drawing color = Lime
#property indicator_color1  Lime
//---- line style = solid line
#property indicator_style1  STYLE_SOLID
//---- line width = 1
#property indicator_width1  1
//---- label of the BullsAroon indicator
#property indicator_label1  "BullsAroon"
//+----------------------------------------------+
//|  Bear indicator plot parameters              |
//+----------------------------------------------+
//---- drawing style = line
#property indicator_type2   DRAW_LINE
//---- drawing color = Red
#property indicator_color2  Red
//---- line style = solid line
#property indicator_style2  STYLE_SOLID
//---- line width = 1
#property indicator_width2  1
//---- label of the BearsAroon indicator
#property indicator_label2  "BearsAroon"
//+----------------------------------------------+
//| Horizontal levels                            |
//+----------------------------------------------+
#property indicator_level1 70.0
#property indicator_level2 50.0
#property indicator_level3 30.0
#property indicator_levelcolor Gray
#property indicator_levelstyle STYLE_DASHDOTDOT
//+----------------------------------------------+
//| Indicator input parameters                   |
//+----------------------------------------------+
input int AroonPeriod = 9; // Period
input int AroonShift = 0;  // Horizontal shift in bars
//+----------------------------------------------+
//--- declare the dynamic arrays used further as indicator buffers
double BullsAroonBuffer[];
double BearsAroonBuffer[]; 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
void OnInit()
  {
//----+
   //--- set BullsAroonBuffer dynamic array as indicator buffer 
   SetIndexBuffer(0, BullsAroonBuffer, INDICATOR_DATA);
   //--- horizontal shift (AroonShift) of the indicator 1
   PlotIndexSetInteger(0, PLOT_SHIFT, AroonShift);
   //--- plot draw begin (AroonPeriod) of the indicator 1
   PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, AroonPeriod);
   //--- label shown in DataWindow
   PlotIndexSetString(0, PLOT_LABEL, "BearsAroon"); 
   
   //--- set BearsAroonBuffer dynamic array as indicator buffer 
   SetIndexBuffer(1, BearsAroonBuffer, INDICATOR_DATA);
   //--- horizontal shift (AroonShift) of the indicator 2 
   PlotIndexSetInteger(1, PLOT_SHIFT, AroonShift);
   //--- plot draw begin (AroonPeriod) of the indicator 2 
   PlotIndexSetInteger(1, PLOT_DRAW_BEGIN, AroonPeriod);
   //--- label shown in DataWindow 
   PlotIndexSetString(1, PLOT_LABEL, "BullsAroon");  
   
   //--- initialization of the variable for a short indicator name
   string shortname;
   StringConcatenate(shortname, "Aroon(", AroonPeriod, ", ", AroonShift, ")"); 
   //--- set short indicator name
   IndicatorSetString(INDICATOR_SHORTNAME, shortname);
   //--- set accuracy of drawing of indicator values
   IndicatorSetInteger(INDICATOR_DIGITS, 0);
//----+
  }
//+------------------------------------------------------------------+
//|  searching index of the highest bar                              |
//+------------------------------------------------------------------+
int iHighest(const double &array[], // array for searching for the index of the maximum element
             int count,            // number of the elements in the array (in the decreasing order), 
             int startPos          // starting index
             )                     
  {
//---+
   int index = startPos;
   
   //---- checking the starting index
   if (startPos < 0)
     {
      Print("Incorrect value in the function iHighest, startPos = ", startPos);
      return (0);
     } 
   //---- checking the startPos values
   if (startPos - count < 0) count = startPos;
    
   double max = array[startPos];
   
   //---- index search
   for(int i = startPos; i > startPos - count; i--)
     {
      if(array[i] > max)
        {
         index = i;
         max = array[i];
        }
     }
//---+ return of the index of the largest bar
   return(index);
  }
//+------------------------------------------------------------------+
//|  searching index of the lowest bar                               |
//+------------------------------------------------------------------+
int iLowest(
            const double &array[], // array for searching for the index of the maximum element
            int count,            // number of the elements in the array (in the decreasing order),
            int startPos          // starting index
            ) 
{
//---+
   int index = startPos;
   
   //--- checking the stating index
   if (startPos < 0)
     {
      Print("Incorrect value in the iLowest function, startPos = ",startPos);
      return(0);
     }
     
   //--- checking the startPos value
   if (startPos - count < 0) count = startPos;
    
   double min = array[startPos];
   
   //--- index search
   for(int i = startPos; i > startPos - count; i--)
     {
      if (array[i] < min)
        {
         index = i;
         min = array[i];
        }
     }
//---+ return of the index of the smallest bar
   return(index);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,    // total bars on the current tick
                const int prev_calculated,// total bars on the previous tick
                const datetime& time[],
                const double& open[],    
                const double& high[],     // price array of the maximum prices for the indicator calculations
                const double& low[],      // price array of the minimum prices for the indicator calculations
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[]
               )
  {
//----+   
   //--- checking the number of bars
   if (rates_total < AroonPeriod - 1)
    return(0);
   
   //--- declare the local variables
   int first, bar;
   double BULLS, BEARS;
   
   //--- calculation of the first (staring index) for the main loop
   if (prev_calculated == 0)   // checking for the first call of the OnCalculate function
     first = AroonPeriod - 1;  // starting index for calculating all of the bars 

   else first = prev_calculated - 1; // starting index for calculating new bars

   //--- main loop
   for(bar = first; bar < rates_total; bar++)
    {
     //--- calculation of values
     BULLS = 100 - (bar - iHighest(high, AroonPeriod, bar) + 0.5) * 100 / AroonPeriod;
     BEARS = 100 - (bar - iLowest (low,  AroonPeriod, bar) + 0.5) * 100 / AroonPeriod;

     //--- filling the indicator buffers with the calculated values 
     BullsAroonBuffer[bar] = BULLS;
     BearsAroonBuffer[bar] = BEARS;
    }
//----+     
   return(rates_total);
  }
//+------------------------------------------------------------------+
