//+------------------------------------------------------------------+
//|                                                       AroonS.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
//---- copyright
#property copyright "2010, MetaQuotes Software Corp."
//---- link to the author's site
#property link      "http://www.mql5.com"
//---- version number
#property version   "1.00"
//---- plot indicator in the separate window
#property indicator_separate_window
//---- two buffers are used
#property indicator_buffers 2
//---- two plots are used
#property indicator_plots   2
//+----------------------------------------------+
//|  Bull indicator plot parameters              |
//+----------------------------------------------+
//---- drawing style = line
#property indicator_type1   DRAW_LINE
//---- drawing color = Lime
#property indicator_color1  Lime
//---- line style = solid line
#property indicator_style1  STYLE_SOLID
//---- line width = 1
#property indicator_width1  1
//---- label of the BullsAroon indicator
#property indicator_label1  "BullsAroon"
//+----------------------------------------------+
//|  Bear indicator plot parameters              |
//+----------------------------------------------+
//---- drawing style = line
#property indicator_type2   DRAW_LINE
//---- drawing color = Red
#property indicator_color2  Red
//---- line style = solid line
#property indicator_style2  STYLE_SOLID
//---- line width = 1
#property indicator_width2  1
//---- label of the BearsAroon indicator
#property indicator_label2  "BearsAroon"
//+----------------------------------------------+
//| Horizontal levels                            |
//+----------------------------------------------+
#property indicator_level1 70.0
#property indicator_level2 50.0
#property indicator_level3 30.0
#property indicator_levelcolor Gray
#property indicator_levelstyle STYLE_DASHDOTDOT
//+----------------------------------------------+
//| Indicator input parameters                   |
//+----------------------------------------------+
input int AroonPeriod = 9; //   
input int AroonShift = 0; //       
//+----------------------------------------------+
input int AroonPeriod = 9; // Period
input int AroonShift = 0;  // Horizontal shift in bars
//+----------------------------------------------+
//--- declare the dynamic arrays used further as indicator buffers
double BullsAroonBuffer[];
double BearsAroonBuffer[]; 

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
void OnInit()
  {
//----+
   //--- set BullsAroonBuffer dynamic array as indicator buffer 
   SetIndexBuffer(0, BullsAroonBuffer, INDICATOR_DATA);
   //--- horizontal shift (AroonShift) of the indicator 1
   PlotIndexSetInteger(0, PLOT_SHIFT, AroonShift);
   //--- plot draw begin (AroonPeriod) of the indicator 1
   PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, AroonPeriod);
   //--- label shown in DataWindow
   PlotIndexSetString(0, PLOT_LABEL, "BearsAroon"); 
   
   //--- set BearsAroonBuffer dynamic array as indicator buffer 
   SetIndexBuffer(1, BearsAroonBuffer, INDICATOR_DATA);
   //--- horizontal shift (AroonShift) of the indicator 2 
   PlotIndexSetInteger(1, PLOT_SHIFT, AroonShift);
   //--- plot draw begin (AroonPeriod) of the indicator 2 
   PlotIndexSetInteger(1, PLOT_DRAW_BEGIN, AroonPeriod);
   //--- label shown in DataWindow 
   PlotIndexSetString(1, PLOT_LABEL, "BullsAroon");  
   
   //--- initialization of the variable for a short indicator name
   string shortname;
   StringConcatenate(shortname, "Aroon(", AroonPeriod, ", ", AroonShift, ")"); 
   //--- set short indicator name
   IndicatorSetString(INDICATOR_SHORTNAME, shortname);
   //--- set accuracy of drawing of indicator values
   IndicatorSetInteger(INDICATOR_DIGITS, 0);
//----+
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,    // total bars on the current tick
                const int prev_calculated,// total bars on the previous tick
                const datetime& time[],
                const double& open[],    
                const double& high[],     // price array of the maximum prices for the indicator calculations
                const double& low[],      // price array of the minimum prices for the indicator calculations
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[]
               )
  {
//----+   
   //---- checking the number of bars
   if (rates_total < AroonPeriod - 1)
    return(0);
    
   //---- set indexation as time series
   ArraySetAsSeries(high, true);
   ArraySetAsSeries(low,  true);
   ArraySetAsSeries(BullsAroonBuffer, true);
   ArraySetAsSeries(BearsAroonBuffer, true);
   
   //---- declare the local variables
   int limit, bar;
   double BULLS, BEARS;
   
   //---- calculation of the first (staring index) for the main loop
   if (prev_calculated == 0)                   // checking for the first call of the OnCalculate function
     limit = rates_total - AroonPeriod - 1;    // starting index for calculating all of the bars 
   else limit = rates_total - prev_calculated; // starting index for calculating new bars
   
   //---- main loop
   for(bar = limit; bar >= 0; bar--)
    {
     //---- calculation of values
     BULLS = 100 + (bar - ArrayMaximum(high, bar, AroonPeriod) - 0.5) * 100 / AroonPeriod;
     BEARS = 100 + (bar - ArrayMinimum(low,  bar, AroonPeriod) - 0.5) * 100 / AroonPeriod;

     //---- filling the indicator buffers with the calculated values
     BullsAroonBuffer[bar] = BULLS;
     BearsAroonBuffer[bar] = BEARS;
    }
//----+     
   return(rates_total);
  }
//+------------------------------------------------------------------+
