//+------------------------------------------------------------------+
//|                                                        SMA_1.mq5 |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
//---- copyright
#property copyright "2010, MetaQuotes Software Corp."
//---- link to the author's site
#property link      "http://www.mql5.com"
//---- version number
#property version   "1.00"
//---- plot indicator in the separate window
#property indicator_chart_window
//---- one buffer is used
#property indicator_buffers 1
//---- one plot is used
#property indicator_plots   1
//---- plotting as line
#property indicator_type1   DRAW_LINE
//---- use red color as the color of the indicator
#property indicator_color1  Red
//---- line style = solid
#property indicator_style1  STYLE_SOLID
//---- line width = 1
#property indicator_width1  1
//---- indicator label
#property indicator_label1  "SMA"

//---- indicator input parameters
input int MAPeriod = 13; // period of moving average
input int MAShift = 0;   // horizontal shift of moving average

//---- declare a dynamic array, used further
// as the buffer of the indicator
double ExtLineBuffer[]; 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
void OnInit()
  {
//----+
   //---- set ExtLineBuffer dynamic array as indicator buffer
   SetIndexBuffer(0, ExtLineBuffer, INDICATOR_DATA);
   //---- horizontal shift 
   PlotIndexSetInteger(0, PLOT_SHIFT, MAShift);
   //---- plot draw begin
   PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, MAPeriod);
   //---- indicator's short name
   string shortname;
   StringConcatenate(shortname, "SMA(", MAPeriod, ",", MAShift, ")");
   //--- label shown in DataWindow
   PlotIndexSetString(0, PLOT_LABEL, shortname);   
   //--- indicator short name
   IndicatorSetString(INDICATOR_SHORTNAME, shortname);
   //--- set accuracy of drawing of indicator values
   IndicatorSetInteger(INDICATOR_DIGITS, _Digits + 1);
//----+
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,    // total bars on the current tick
                const int prev_calculated,// total bars on the previous tick
                const int begin,          // begin position
                const double &price[]     // price array
               )
  {
//----+   
   //---- checking number of bars
   if (rates_total < MAPeriod - 1 + begin)
    return(0);
   
   //---- declare the local variables
   int first, bar, iii;
   double Sum, SMA;
   
   //---- calculation of starting index
   if (prev_calculated == 0) // check for the first call of the OnCalculate function
    { 
     first = MAPeriod - 1 + begin; // starting index for all bars
     //--- increase the position of data beginning by begin bars 
     if(begin > 0)
      PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, begin + MAPeriod);
    }
   else first = prev_calculated - 1; // starting index for new bars

   //---- indicator main loop
   for(bar = first; bar < rates_total; bar++)
    {
     Sum = 0.0;
     //---- price summation
     for(iii = 0; iii < MAPeriod; iii++)
      Sum += price[bar - iii]; // Sum = Sum + price[bar - iii]; // the same
     
     //---- calculation of average price
     SMA = Sum / MAPeriod;
     
     //---- filling the buffer with calculated value
     ExtLineBuffer[bar] = SMA;
    }
//----+     
   return(rates_total);
  }
//+------------------------------------------------------------------+
